@@ -29,6 +29,7 @@ import java.awt.*;
 import java.awt.event.*;
 import java.io.*;
 import java.net.*;
+import java.util.*;
 
 public class VncViewer extends java.applet.Applet
   implements java.lang.Runnable, WindowListener {
@@ -94,12 +95,14 @@ public class VncViewer extends java.applet.Applet
   String urlPrefix;
   String httpsPort;
   String oneTimeKey;
+  String serverCert;
   String proxyHost;
   String proxyPort;
   boolean forceProxy;
   boolean ignoreProxy;
   boolean trustAllVncCerts;
   boolean trustUrlVncCert;
+  boolean debugCerts;
 
   // Reference to this applet for inter-applet communication.
   public static java.applet.Applet refApplet;
@@ -295,10 +298,24 @@ public class VncViewer extends java.applet.Applet
       validate();
     }
 
-    while (!tryAuthenticate()) {
-      authenticator.retry();
-      authenticatorUnixLogin.retry();
-    }
+	if (false) {
+		/* a bug on retries: 'Error: bad position: 8' sun.awt.X11.XTextFieldPeer.setCaretPosition(XTextFieldPeer.java:215) */
+		while (!tryAuthenticate()) {
+			authenticator.retry();
+			authenticatorUnixLogin.retry();
+		}
+	} else {
+		/* just try once and not forever... */
+		if (!tryAuthenticate()) {
+    			showConnectionStatus("Authentication Failed.");
+			showMessage("Authentication Failed.");
+			if (!offerRelogin) {
+				fatalError("auth failed.");
+			}
+		} else {
+			//showConnectionStatus("Authentication OK.");
+		}
+	}
   }
 
 
@@ -441,7 +458,10 @@ public class VncViewer extends java.applet.Applet
     gbc.ipadx = 100;
     gbc.ipady = 50;
     gridbag.setConstraints(authPanel, gbc);
+    try {
     vncContainer.add(authPanel);
+    } catch (Exception e) {
+    }
 
     if (inSeparateFrame) {
       vncFrame.pack();
@@ -603,6 +623,8 @@ public class VncViewer extends java.applet.Applet
 	fatalError("HOST parameter not specified");
       }
     }
+    Date d = new Date();
+    System.out.println("-\nSSL VNC Java Applet starting.  " + d);
 
     port = 0;
     String str = readParameter("PORT", false);
@@ -695,7 +717,12 @@ public class VncViewer extends java.applet.Applet
 
     oneTimeKey = readParameter("oneTimeKey", false);
     if (oneTimeKey != null) {
-    	System.out.println("oneTimeKey: is set");
+    	System.out.println("oneTimeKey is set.");
+    }
+
+    serverCert = readParameter("serverCert", false);
+    if (serverCert != null) {
+    	System.out.println("serverCert is set.");
     }
 
     forceProxy = false;
@@ -746,6 +773,11 @@ public class VncViewer extends java.applet.Applet
     if (str != null && str.equalsIgnoreCase("Yes")) {
 	trustUrlVncCert = true;
     }
+    debugCerts = false;
+    str = readParameter("debugCerts", false);
+    if (str != null && str.equalsIgnoreCase("Yes")) {
+	debugCerts = true;
+    }
   }
 
   public String readParameter(String name, boolean required) {
