@@ -41,6 +41,7 @@ import java.awt.event.*;
 import java.io.*;
 import java.net.*;
 import javax.swing.*;
+import java.util.Date;
 
 public class VncViewer extends java.applet.Applet
   implements java.lang.Runnable, WindowListener {
@@ -122,10 +123,15 @@ String CONNECT;
 String urlPrefix;
 String httpsPort;
 String oneTimeKey;
+String serverCert;
+String ftpDropDown;
+String proxyHost;
+String proxyPort;
 boolean forceProxy;
 boolean ignoreProxy;
 boolean trustAllVncCerts;
 boolean trustUrlVncCert;
+boolean debugCerts;
 
 boolean ignoreMSLogonCheck;
 boolean delayAuthPanel;
@@ -468,6 +474,7 @@ public void ftp_init() {
      //mslogon support end
     }
 
+    int tries = 0;
     while (true) {
       // Wait for user entering a password, or a username and a password
       synchronized(authenticator) {
@@ -488,7 +495,10 @@ public void ftp_init() {
       //mslogon support end
 
 // begin runge/x11vnc
-	gotAuth = false;
+      gotAuth = false;
+      if (++tries > 2) {
+         throw new Exception("Incorrect password entered " + tries + " times.");
+      }
 // end runge/x11vnc
 
       // Retry on authentication failure.
@@ -901,6 +911,8 @@ System.out.println("as: " + authScheme);
 	fatalError("HOST parameter not specified");
       }
     }
+    Date d = new Date();
+    System.out.println("-\nSSL VNC Java Applet starting.  " + d);
 
     port = 0;
     String str = readParameter("PORT", false);
@@ -986,21 +998,64 @@ System.out.println("as: " + authScheme);
     }
     System.out.println("urlPrefix: '" + urlPrefix + "'");
 
+    ftpDropDown = readParameter("ftpDropDown", false);
+    if (ftpDropDown != null) {
+	ftpDropDown = ftpDropDown.replaceAll("%2F", "/");
+	ftpDropDown = ftpDropDown.replaceAll("%2f", "/");
+	ftpDropDown = ftpDropDown.replaceAll("_2F_", "/");
+	ftpDropDown = ftpDropDown.replaceAll("%20", " ");
+	System.out.println("ftpDropDown: '" + ftpDropDown + "'");
+    }
+
+
     oneTimeKey = readParameter("oneTimeKey", false);
     if (oneTimeKey != null) {
-    	System.out.println("oneTimeKey: is set");
+    	System.out.println("oneTimeKey is set.");
+    }
+
+    serverCert = readParameter("serverCert", false);
+    if (serverCert != null) {
+    	System.out.println("serverCert is set.");
     }
 
     forceProxy = false;
+    proxyHost = null;
+    proxyPort = null;
     str = readParameter("forceProxy", false);
-    if (str != null && str.equalsIgnoreCase("Yes")) {
-	forceProxy = true;
+    if (str != null) {
+	    if (str.equalsIgnoreCase("Yes")) {
+		forceProxy = true;
+	    } else if (str.equalsIgnoreCase("No")) {
+		forceProxy = false;
+	    } else {
+		forceProxy = true;
+		String[] pieces = str.split(" ");
+		proxyHost = new String(pieces[0]);
+		if (pieces.length >= 2) {
+			proxyPort = new String(pieces[1]);
+		} else {
+			proxyPort = new String("8080");
+		}
+	    }
     }
+    str = readParameter("proxyHost", false);
+    if (str != null) {
+	proxyHost = new String(str);
+    }
+    str = readParameter("proxyPort", false);
+    if (str != null) {
+	proxyPort = new String(str);
+    }
+    if (proxyHost != null && proxyPort == null) {
+    	proxyPort = new String("8080");
+    }
+
     ignoreProxy = false;
     str = readParameter("ignoreProxy", false);
     if (str != null && str.equalsIgnoreCase("Yes")) {
 	ignoreProxy = true;
     }
+
     trustAllVncCerts = false;
     str = readParameter("trustAllVncCerts", false);
     if (str != null && str.equalsIgnoreCase("Yes")) {
@@ -1011,6 +1066,11 @@ System.out.println("as: " + authScheme);
     if (str != null && str.equalsIgnoreCase("Yes")) {
 	trustUrlVncCert = true;
     }
+    debugCerts = false;
+    str = readParameter("debugCerts", false);
+    if (str != null && str.equalsIgnoreCase("Yes")) {
+	debugCerts = true;
+    }
     ignoreMSLogonCheck = false;
     str = readParameter("ignoreMSLogonCheck", false);
     if (str != null && str.equalsIgnoreCase("Yes")) {
