@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2009 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -789,6 +789,7 @@ char find_display[] =
 "	exit 1\n"
 "fi\n"
 "\n"
+"# Set PATH to pick up utilities we use below.\n"
 "PATH=$PATH:/bin:/usr/bin:/usr/X11R6/bin:/usr/bin/X11:/usr/openwin/bin:/usr/ucb\n"
 "export PATH\n"
 "\n"
@@ -1537,6 +1538,9 @@ char create_display[] =
 "	elif [ \"X$have_startkde\" != \"X\"    -a \"X$FD_SESS\" = \"Xkde\" ]; then\n"
 "		echo \"$have_startkde\"\n"
 "		return\n"
+"	elif [ \"X$have_startlxde\" != \"X\"    -a \"X$FD_SESS\" = \"Xlxde\" ]; then\n"
+"		echo \"$have_startlxde\"\n"
+"		return\n"
 "	elif [ \"X$have_twm\" != \"X\"         -a \"X$FD_SESS\" = \"Xtwm\" ]; then\n"
 "		echo \"$have_twm\"\n"
 "		return\n"
@@ -1600,11 +1604,17 @@ char create_display[] =
 "	fi\n"
 "	if [ -f \"$home/.dmrc\" ]; then\n"
 "		if [ \"X$have_startkde\" != \"X\" ]; then\n"
-"			if egrep -i 'Session=(default|kde)' \"$home/.dmrc\" > /dev/null; then\n"
+"			if egrep -i 'Session=kde' \"$home/.dmrc\" > /dev/null; then\n"
 "				echo \"$have_startkde\"\n"
 "				return\n"
 "			fi\n"
 "		fi\n"
+"		if [ \"X$have_startlxde\" != \"X\" ]; then\n"
+"			if egrep -i 'Session=lxde' \"$home/.dmrc\" > /dev/null; then\n"
+"				echo \"$have_startlxde\"\n"
+"				return\n"
+"			fi\n"
+"		fi\n"
 "		if [ \"X$have_gnome_session\" != \"X\" ]; then\n"
 "			if egrep -i 'Session=gnome' \"$home/.dmrc\" > /dev/null; then\n"
 "				echo \"$have_gnome_session\"\n"
@@ -1623,6 +1633,18 @@ char create_display[] =
 "			fi\n"
 "			\n"
 "		done\n"
+"		if egrep -i 'Session=default' \"$home/.dmrc\" > /dev/null; then\n"
+"			if [ \"X$have_gnome_session\" != \"X\" ]; then\n"
+"				echo \"$have_gnome_session\"\n"
+"				return\n"
+"			elif [ \"X$have_startkde\" != \"X\" ]; then\n"
+"				echo \"$have_startkde\"\n"
+"				return\n"
+"			elif [ \"X$have_startxfce\" != \"X\" ]; then\n"
+"				echo \"$have_startxfce\"\n"
+"				return\n"
+"			fi\n"
+"		fi\n"
 "	fi\n"
 "	if [ -f \"$home/.xsession\" ]; then\n"
 "		echo \"$home/.xsession\"\n"
@@ -1779,6 +1801,13 @@ char create_display[] =
 "	echo \"$tf\"\n"
 "}\n"
 "\n"
+"missing_mesg() {\n"
+"	echo \"\" 1>&2\n"
+"	echo \"The program \\\"$1\\\" could not be found in PATH and standard locations.\" 1>&2\n"
+"	echo \"You probably need to install a package that provides the \\\"$1\\\" program.\" 1>&2\n"
+"	echo \"\" 1>&2\n"
+"}\n"
+"\n"
 "server() {\n"
 "	authfile=`auth`\n"
 "	sess=`findsession`\n"
@@ -1802,6 +1831,10 @@ char create_display[] =
 "		chmod 755 $stmp || exit 1\n"
 "		echo \"#!/bin/sh\" > $stmp\n"
 "		#echo \"(id; env; env | grep XAUTHORITY | sed -e 's/XAUTHORITY=//' | xargs ls -l) > /tmp/ENV.OUT.$$\" >> $stmp\n"
+"		if [ \"X$SAVE_PATH\" != \"X\" ]; then\n"
+"			echo \"PATH=\\\"$SAVE_PATH\\\"\" >> $stmp\n"
+"			echo \"export PATH\" >> $stmp\n"
+"		fi\n"
 "		if [ \"X$noxauth\" = \"X1\" ]; then\n"
 "			echo \"unset XAUTHORITY\"   >> $stmp\n"
 "		fi\n"
@@ -1842,19 +1875,30 @@ char create_display[] =
 "\n"
 "	if [ \"X$use_xdmcp_query\" = \"X1\" ]; then\n"
 "		# we cannot use -nolisten tcp\n"
-"		echo \"$* -once -query localhost $FD_OPTS\" 1>&2\n"
+"		if [ \"X$FD_XDMCP_IF\" != \"X\" ]; then\n"
+"			lhost=$FD_XDMCP_IF\n"
+"		elif [ \"X$have_netstat\" = \"X\" ]; then\n"
+"			lhost=localhost\n"
+"		elif $have_netstat -an | grep -w 177 | grep -w udp > /dev/null; then\n"
+"			lhost=localhost\n"
+"		elif $have_netstat -an | grep -w 177 | grep -w udp6 > /dev/null; then\n"
+"			lhost=::1\n"
+"		else\n"
+"			lhost=localhost\n"
+"		fi\n"
+"		echo \"$* -once -query $lhost $FD_OPTS\" 1>&2\n"
 "		if [ \"X$have_root\" != \"X\" ]; then\n"
 "			if [ -r $authfile ]; then\n"
-"				$have_nohup $* -once -query localhost -auth $authfile $FD_OPTS 1>&2 &\n"
+"				$have_nohup $* -once -query $lhost -auth $authfile $FD_OPTS 1>&2 &\n"
 "			else\n"
 "				# why did we have this?\n"
-"				$have_nohup $* -once -query localhost $FD_OPTS 1>&2 &\n"
+"				$have_nohup $* -once -query $lhost $FD_OPTS 1>&2 &\n"
 "			fi\n"
 "		else\n"
 "			if [ \"X$ns\" = \"X0\" ]; then\n"
-"				$have_nohup sh -c \"$* -once -query localhost -auth $authfile $FD_OPTS\" 1>&2 &\n"
+"				$have_nohup sh -c \"$* -once -query $lhost -auth $authfile $FD_OPTS\" 1>&2 &\n"
 "			else\n"
-"				$have_nohup sh -c \"(sleep $ns; $* -once -query localhost -auth $authfile $FD_OPTS)\" 1>&2 &\n"
+"				$have_nohup sh -c \"(sleep $ns; $* -once -query $lhost -auth $authfile $FD_OPTS)\" 1>&2 &\n"
 "				#result=1\n"
 "			fi\n"
 "		fi\n"
@@ -1939,25 +1983,6 @@ char create_display[] =
 "		fi\n"
 "	done\n"
 "\n"
-"#	if [ \"X$result\" = \"X1\" ]; then\n"
-"#		if [ \"X$use_xdmcp_query\" = \"X0\" -a \"X$have_xdpyinfo\" != \"X\" ]; then\n"
-"#			ok=0\n"
-"#			for t in 1 2 3 4\n"
-"#			do\n"
-"#				$have_xdpyinfo >/dev/null 2>&1\n"
-"#				if [ $? != 0 ]; then\n"
-"#					sleep 1\n"
-"#				else\n"
-"#					ok=1\n"
-"#					break;\n"
-"#				fi\n"
-"#			done\n"
-"#			if [ \"X$ok\" = \"X0\" ]; then\n"
-"#				result=0\n"
-"#			fi\n"
-"#		fi\n"
-"#	fi\n"
-"\n"
 "	if [ \"X$redir_daemon\" != \"X\" -a \"X$result\" = \"X1\" ]; then\n"
 "		redir_daemon=`echo \"$redir_daemon\" | sed -e 's/[~!$&*()|;?<>\"]//g' -e \"s/'//g\"`\n"
 "		xprog=$X11VNC_PROG\n"
@@ -1990,21 +2015,9 @@ char create_display[] =
 "	fi\n"
 "}\n"
 "\n"
-"try_Xdummy() {\n"
-"	if [ \"X$have_Xdummy\" = \"X\" ]; then\n"
-"		return\n"
-"	fi\n"
-"	if [ \"X$FD_XDUMMY_NOROOT\" != \"X\" ]; then\n"
-"		:\n"
-"	elif [ \"X$have_root\" = \"X\" ]; then\n"
-"		return\n"
-"	fi\n"
-"\n"
-"	server $have_Xdummy :$N -geometry $geom -depth $depth\n"
-"}\n"
-"\n"
 "try_Xvnc() {\n"
 "	if [ \"X$have_Xvnc\" = \"X\" ]; then\n"
+"		missing_mesg Xvnc\n"
 "		return\n"
 "	fi\n"
 "\n"
@@ -2019,9 +2032,61 @@ char create_display[] =
 "	server $FD_XSRV :$N -geometry $geom -depth $depth\n"
 "}\n"
 "\n"
+"add_modmap() {\n"
+"	if [ \"X$have_root\" = \"X\" ]; then\n"
+"	    $have_nohup sh -c \"(\n"
+"		sleep 10;\n"
+"		$have_xmodmap -display :$N -e 'keycode any = Shift_R' \\\n"
+"			-e 'add Shift = Shift_L Shift_R' \\\n"
+"			-e 'keycode any = Control_R' \\\n"
+"			-e 'add Control = Control_L Control_R' \\\n"
+"			-e 'keycode any = Alt_L' \\\n"
+"			-e 'keycode any = Alt_R' \\\n"
+"			-e 'keycode any = Meta_L' \\\n"
+"			-e 'clear Mod1' \\\n"
+"			-e 'add Mod1 = Alt_L Alt_R Meta_L';\n"
+"		if uname | grep SunOS > /dev/null; then\n"
+"			for sym in SunAudioMute SunAudioLowerVolume SunAudioRaiseVolume\n"
+"			do\n"
+"				if $have_xmodmap -pk | grep -w \\$sym > /dev/null; then\n"
+"					:\n"
+"				else\n"
+"					$have_xmodmap -e \\\"keycode any = \\$sym\\\"\n"
+"				fi\n"
+"			done\n"
+"		fi\n"
+"\n"
+"	    )\" 1>&2 &\n"
+"	else\n"
+"	    (\n"
+"		sleep 6;\n"
+"		$have_xmodmap -display :$N -e 'keycode any = Shift_R' \\\n"
+"			-e 'add Shift = Shift_L Shift_R' \\\n"
+"			-e 'keycode any = Control_R' \\\n"
+"			-e 'add Control = Control_L Control_R' \\\n"
+"			-e 'keycode any = Alt_L' \\\n"
+"			-e 'keycode any = Alt_R' \\\n"
+"			-e 'keycode any = Meta_L' \\\n"
+"			-e 'clear Mod1' \\\n"
+"			-e 'add Mod1 = Alt_L Alt_R Meta_L';\n"
+"		# this is to workaround a bug with JDS Solaris 10 gnome-session-daemon.\n"
+"		if uname | grep SunOS > /dev/null; then\n"
+"			for sym in SunAudioMute SunAudioLowerVolume SunAudioRaiseVolume\n"
+"			do\n"
+"				if $have_xmodmap -pk | grep -w $sym > /dev/null; then\n"
+"					:\n"
+"				else\n"
+"					$have_xmodmap -e \"keycode any = $sym\"\n"
+"				fi\n"
+"			done\n"
+"		fi\n"
+"	    ) 1>&2 &\n"
+"	fi\n"
+"}\n"
 "\n"
 "try_Xvfb() {\n"
 "	if [ \"X$have_Xvfb\" = \"X\" ]; then\n"
+"		missing_mesg Xvfb\n"
 "		return\n"
 "	fi\n"
 "\n"
@@ -2031,7 +2096,10 @@ char create_display[] =
 "			sarg=\"screen\"\n"
 "		fi\n"
 "	fi\n"
-"	margs=\"+kb\"\n"
+"	margs=\"\"\n"
+"	if $have_Xvfb -help 2>&1 | grep '^\\+kb[ 	].*Keyboard Extension' >/dev/null; then\n"
+"		margs=\"+kb\"\n"
+"	fi\n"
 "\n"
 "	# currently not enabled in Xvfb's we see.\n"
 "#	if $have_Xvfb -extension MOOMOO 2>&1 | grep -w RANDR >/dev/null; then\n"
@@ -2045,58 +2113,28 @@ char create_display[] =
 "	server $have_Xvfb :$N $sarg 0 ${geom}x${depth} $margs\n"
 "\n"
 "	if [ \"X$result\" = \"X1\" -a \"X$have_xmodmap\" != \"X\" ]; then\n"
-"		if [ \"X$have_root\" = \"X\" ]; then\n"
-"		    $have_nohup sh -c \"(\n"
-"			sleep 10;\n"
-"			$have_xmodmap -display :$N -e 'keycode any = Shift_R' \\\n"
-"				-e 'add Shift = Shift_L Shift_R' \\\n"
-"				-e 'keycode any = Control_R' \\\n"
-"				-e 'add Control = Control_L Control_R' \\\n"
-"				-e 'keycode any = Alt_L' \\\n"
-"				-e 'keycode any = Alt_R' \\\n"
-"				-e 'keycode any = Meta_L' \\\n"
-"				-e 'clear Mod1' \\\n"
-"				-e 'add Mod1 = Alt_L Alt_R Meta_L';\n"
-"			if uname | grep SunOS > /dev/null; then\n"
-"				for sym in SunAudioMute SunAudioLowerVolume SunAudioRaiseVolume\n"
-"				do\n"
-"					if $have_xmodmap -pk | grep -w \\$sym > /dev/null; then\n"
-"						:\n"
-"					else\n"
-"						$have_xmodmap -e \\\"keycode any = \\$sym\\\"\n"
-"					fi\n"
-"				done\n"
-"			fi\n"
-"\n"
-"		    )\" 1>&2 &\n"
-"		else\n"
-"		    (\n"
-"			sleep 6;\n"
-"			$have_xmodmap -display :$N -e 'keycode any = Shift_R' \\\n"
-"				-e 'add Shift = Shift_L Shift_R' \\\n"
-"				-e 'keycode any = Control_R' \\\n"
-"				-e 'add Control = Control_L Control_R' \\\n"
-"				-e 'keycode any = Alt_L' \\\n"
-"				-e 'keycode any = Alt_R' \\\n"
-"				-e 'keycode any = Meta_L' \\\n"
-"				-e 'clear Mod1' \\\n"
-"				-e 'add Mod1 = Alt_L Alt_R Meta_L';\n"
-"			# this is to workaround a bug with JDS Solaris 10 gnome-session-daemon.\n"
-"			if uname | grep SunOS > /dev/null; then\n"
-"				for sym in SunAudioMute SunAudioLowerVolume SunAudioRaiseVolume\n"
-"				do\n"
-"					if $have_xmodmap -pk | grep -w $sym > /dev/null; then\n"
-"						:\n"
-"					else\n"
-"						$have_xmodmap -e \"keycode any = $sym\"\n"
-"					fi\n"
-"				done\n"
-"			fi\n"
-"		    ) 1>&2 &\n"
-"		fi\n"
+"		add_modmap\n"
 "	fi\n"
 "}\n"
 "\n"
+"try_Xdummy() {\n"
+"	if [ \"X$have_Xdummy\" = \"X\" ]; then\n"
+"		missing_mesg Xdummy\n"
+"		return\n"
+"	fi\n"
+"	if [ \"X$FD_XDUMMY_RUN_AS_ROOT\" != \"X\" -a \"X$have_root\" = \"X\" ]; then\n"
+"		return\n"
+"	fi\n"
+"\n"
+"	server $have_Xdummy :$N -geometry $geom -depth $depth\n"
+"	\n"
+"	if [ \"X$result\" = \"X1\" -a \"X$have_xprop\" != \"X\" ]; then\n"
+"		(sleep 1; $have_xprop -display :$N -root -f X11VNC_TRAP_XRANDR 8s -set X11VNC_TRAP_XRANDR 1 >/dev/null 2>&1) &\n"
+"		sleep 1\n"
+"	fi\n"
+"}\n"
+"\n"
+"\n"
 "cookie() {\n"
 "	cookie=\"\"\n"
 "	if [ \"X$have_mcookie\" != \"X\" ]; then\n"
@@ -2234,7 +2272,9 @@ char create_display[] =
 "	USER=`whoami`\n"
 "fi\n"
 "\n"
-"PATH=$PATH:/usr/X11R6/bin:/usr/bin/X11:/usr/openwin/bin:/usr/dt/bin:/opt/kde4/bin:/opt/kde3/bin:/opt/gnome/bin:/usr/bin:/bin:/usr/sfw/bin\n"
+"# Set PATH to have a better chance of finding things:\n"
+"SAVE_PATH=$PATH\n"
+"PATH=$PATH:/usr/X11R6/bin:/usr/bin/X11:/usr/openwin/bin:/usr/dt/bin:/opt/kde4/bin:/opt/kde3/bin:/opt/gnome/bin:/usr/bin:/bin:/usr/sfw/bin:/usr/local/bin\n"
 "\n"
 "have_root=\"\"\n"
 "id0=`id`\n"
@@ -2247,7 +2287,7 @@ char create_display[] =
 "	p_ok=1\n"
 "fi\n"
 "\n"
-"for prog in startx xinit xdm gdm kdm xterm Xdummy Xvfb Xvnc xauth xdpyinfo mcookie md5sum xmodmap startkde dbus-launch gnome-session blackbox fvwm2 mwm openbox twm windowmaker wmaker enlightenment metacity X Xorg XFree86 Xsun Xsession dtwm netstat nohup esddsp konsole gnome-terminal x-terminal-emulator perl startxfce4 startxfce\n"
+"for prog in startx xinit xdm gdm kdm xterm Xdummy Xvfb Xvnc xauth xdpyinfo mcookie md5sum xmodmap startkde startlxde dbus-launch gnome-session blackbox fvwm2 mwm openbox twm windowmaker wmaker enlightenment metacity X Xorg XFree86 Xsun Xsession dtwm netstat nohup esddsp konsole gnome-terminal x-terminal-emulator perl startxfce4 startxfce xprop\n"
 "do\n"
 "	p2=`echo \"$prog\" | sed -e 's/-/_/g'`\n"
 "	eval \"have_$p2=''\"\n"
