@@ -1,3 +1,35 @@
+/*
+   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   All rights reserved.
+
+This file is part of x11vnc.
+
+x11vnc is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or (at
+your option) any later version.
+
+x11vnc is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with x11vnc; if not, write to the Free Software
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
+or see <http://www.gnu.org/licenses/>.
+
+In addition, as a special exception, Karl J. Runge
+gives permission to link the code of its release of x11vnc with the
+OpenSSL project's "OpenSSL" library (or with modified versions of it
+that use the same license as the "OpenSSL" library), and distribute
+the linked executables.  You must obey the GNU General Public License
+in all respects for all of the code used other than "OpenSSL".  If you
+modify this file, you may extend this exception to your version of the
+file, but you are not obligated to do so.  If you do not wish to do
+so, delete this exception statement from your version.
+*/
+
 /* -- xrecord.c -- */
 
 #include "x11vnc.h"
@@ -135,8 +167,8 @@ static void xrecord_grabserver(int start) {
 		XSetErrorHandler(old_handler);
 		return;
 	}
-	XSetErrorHandler(old_handler);
 	XFlush_wr(gdpy_data);
+	XSetErrorHandler(old_handler);
 #else
 	if (!rc || !old_handler) {}
 #endif
@@ -503,12 +535,13 @@ static double xrecord_start = 0.0;
 static void record_CA(XPointer ptr, XRecordInterceptData *rec_data) {
 	xCopyAreaReq *req;
 	Window src = None, dst = None, c;
-	XWindowAttributes attr;
-	int src_x, src_y, dst_x, dst_y, rx, ry;
-	int good = 1, dx, dy, k=0, i;
+	XWindowAttributes attr, attr2;
+	int src_x, src_y, dst_x, dst_y, rx, ry, rx2, ry2;
+	int good = 1, dx = 0, dy = 0, k=0, i;
 	unsigned int w, h;
 	int dba = 0, db = debug_scroll;
 	int cache_index, next_index, valid;
+	static int must_equal = -1;
 
 	if (dba || db) {
 		if (rec_data->category == XRecordFromClient) {
@@ -552,6 +585,13 @@ if (db > 1) fprintf(stderr, "record_CA-%d\n", k++);
 	}
 if (db > 1) fprintf(stderr, "record_CA-%d\n", k++);
 
+	if (must_equal < 0) {
+		must_equal = 0;
+		if (getenv("X11VNC_SCROLL_MUST_EQUAL")) {
+			must_equal = 1;
+		}
+	}
+
 /*
 
 xterm, gnome-terminal, others.
@@ -586,25 +626,36 @@ short period of time with a painting error: two cursors, one above the other.
 	h = req->height;
 
 	if (w*h < (unsigned int) scrollcopyrect_min_area) {
+		if (db > 1) fprintf(stderr, "record_CA scroll area too small.\n");
 		good = 0;
 	} else if (!src || !dst) {
-		good = 0;
-	} else if (src != dst) {
+		if (db > 1) fprintf(stderr, "record_CA null src or dst.\n");
 		good = 0;
 	} else if (scr_ev_cnt >= SCR_EV_MAX) {
+		if (db > 1) fprintf(stderr, "record_CA null too many scr events.\n");
+		good = 0;
+	} else if (must_equal && src != dst) {
+		if (db > 1) fprintf(stderr, "record_CA src not equal dst.\n");
 		good = 0;
 	}
 
-	dx = dst_x - src_x;
-	dy = dst_y - src_y;
+	if (src == dst) {
+		dx = dst_x - src_x;
+		dy = dst_y - src_y;
 
-	if (dx != 0 && dy != 0) {
-		good = 0;
+		if (dx != 0 && dy != 0) {
+			good = 0;
+		}
 	}
 
+if (!good && (dba || db > 1)) fprintf(stderr, "record_CA-x src_x: %d src_y: %d "
+	"dst_x: %d dst_y: %d w: %d h: %d scr_ev_cnt: %d 0x%lx/0x%lx\n",
+	src_x, src_y, dst_x, dst_y, w, h, scr_ev_cnt, src, dst);
+
 	if (! good) {
 		return;
 	}
+
 if (db > 1) fprintf(stderr, "record_CA-%d\n", k++);
 
 	/*
@@ -655,14 +706,84 @@ if (db > 1) fprintf(stderr, "record_CA-%d\n", k++);
 	}
 
 	if (! valid) {
+		if (db > 1) fprintf(stderr, "record_CA not valid-1.\n");
 		return;
 	}
 if (db > 1) fprintf(stderr, "record_CA-%d\n", k++);
 
 	if (attr.map_state != IsViewable) {
+		if (db > 1) fprintf(stderr, "record_CA not viewable-1.\n");
 		return;
 	}
 
+	/* recent gdk/gtk windows use different src and dst. for compositing? */
+	if (src != dst) {
+	    if (lookup_attr_cache(dst, &cache_index, &next_index)) {
+		i = cache_index;
+		attr2.x = scr_attr_cache[i].x;
+		attr2.y = scr_attr_cache[i].y;
+		attr2.width = scr_attr_cache[i].width;
+		attr2.height = scr_attr_cache[i].height;
+		attr2.map_state = scr_attr_cache[i].map_state;
+		rx2 = scr_attr_cache[i].rx;
+		ry2 = scr_attr_cache[i].ry;
+		valid = scr_attr_cache[i].valid;
+
+	    } else {
+		valid = valid_window(dst, &attr2, 1);
+
+		if (valid) {
+			if (!xtranslate(dst, rootwin, 0, 0, &rx2, &ry2, &c, 1)) {
+				valid = 0;
+			}
+		}
+		if (next_index >= 0) {
+			i = next_index;
+			scr_attr_cache[i].win = dst;
+			scr_attr_cache[i].fetched = 1;
+			scr_attr_cache[i].valid = valid;
+			scr_attr_cache[i].time = dnow();
+			if (valid) {
+				scr_attr_cache[i].x = attr2.x;
+				scr_attr_cache[i].y = attr2.y;
+				scr_attr_cache[i].width = attr2.width;
+				scr_attr_cache[i].height = attr2.height;
+				scr_attr_cache[i].border_width = attr2.border_width;
+				scr_attr_cache[i].depth = attr2.depth;
+				scr_attr_cache[i].class = attr2.class;
+				scr_attr_cache[i].backing_store =
+				    attr2.backing_store;
+				scr_attr_cache[i].map_state = attr2.map_state;
+
+				scr_attr_cache[i].rx = rx2;
+				scr_attr_cache[i].ry = ry2;
+			}
+		}
+	    }
+
+if (dba || db > 1) fprintf(stderr, "record_CA-? src_x: %d src_y: %d "
+	"dst_x: %d dst_y: %d w: %d h: %d scr_ev_cnt: %d 0x%lx/0x%lx\n",
+	src_x, src_y, dst_x, dst_y, w, h, scr_ev_cnt, src, dst);
+
+		if (! valid) {
+			if (db > 1) fprintf(stderr, "record_CA not valid-2.\n");
+			return;
+		}
+		if (attr2.map_state != IsViewable) {
+			if (db > 1) fprintf(stderr, "record_CA not viewable-2.\n");
+			return;
+		}
+		dst_x = dst_x - (rx - rx2);
+		dst_y = dst_y - (ry - ry2);
+
+		dx = dst_x - src_x;
+		dy = dst_y - src_y;
+
+		if (dx != 0 && dy != 0) {
+			return;
+		}
+	}
+
 
  if (0 || dba || db) {
 	double st, dt;
@@ -1514,14 +1635,30 @@ void check_xrecord_reset(int force) {
 #endif
 }
 
-#define RECORD_ERROR_MSG \
+#define RECORD_ERROR_MSG(tag) \
 	if (! quiet) { \
-		rfbLog("trapped RECORD XError: %s %d/%d/%d (0x%lx)\n", \
-		    xerror_string(trapped_record_xerror_event), \
-		    (int) trapped_record_xerror_event->error_code, \
-		    (int) trapped_record_xerror_event->request_code, \
-		    (int) trapped_record_xerror_event->minor_code, \
-		    (int) trapped_record_xerror_event->resourceid); \
+		static int cnt = 0; \
+		static time_t last = 0; \
+		int show = 0; \
+		cnt++; \
+		if (debug_scroll || cnt < 20) { \
+			show = 1; \
+		} else if (cnt == 20) { \
+			last = time(NULL); \
+			rfbLog("disabling RECORD XError messages for 600s\n"); \
+			show = 1; \
+		} else if (time(NULL) > last + 600) { \
+			cnt = 0; \
+			show = 1; \
+		} \
+		if (show) { \
+			rfbLog("trapped RECORD XError: %s %s %d/%d/%d (0x%lx)\n", \
+			    tag, xerror_string(trapped_record_xerror_event), \
+			    (int) trapped_record_xerror_event->error_code, \
+			    (int) trapped_record_xerror_event->request_code, \
+			    (int) trapped_record_xerror_event->minor_code, \
+			    (int) trapped_record_xerror_event->resourceid); \
+		} \
 	}
 
 void xrecord_watch(int start, int setby) {
@@ -1545,10 +1682,20 @@ if (0) db = 1;
 		return;
 	}
 	if (use_threads) {
-		/* XXX not working */
-		use_xrecord = 0;
-		xrecording = 0;
-		return;
+		/* XXX not working.  Still?  Painting errors. */
+		static int first = 1;
+		if (first) {
+			if (use_xrecord && !getenv("XRECORD_THREADS")) {
+				rfbLog("xrecord_watch: disabling scroll detection in -threads mode.\n");
+				rfbLog("xrecord_watch: Set -env XRECORD_THREADS=1 to enable it.\n");
+				use_xrecord = 0;
+				xrecording = 0;
+			}
+			first = 0;
+		}
+		if (!use_xrecord && !xrecording) {
+			return;
+		}
 	}
 
 	dtime0(&now);
@@ -1617,7 +1764,7 @@ if (db > 1) fprintf(stderr, "=== shutdown-scroll 0x%lx\n", rc_scroll);
 			XRecordProcessReplies(rdpy_data);
 
 			if (trapped_record_xerror) {
-				RECORD_ERROR_MSG;
+				RECORD_ERROR_MSG("shutdown");
 				last_error = now;
 			}
 
@@ -1641,7 +1788,7 @@ if (db > 1) fprintf(stderr, "=== disab-scroll 0x%lx 0x%lx\n", rc_scroll, rcs_scr
 				XRecordProcessReplies(rdpy_data);
 
 				if (trapped_record_xerror) {
-					RECORD_ERROR_MSG;
+					RECORD_ERROR_MSG("disable");
 
 					shutdown_record_context(rc_scroll,
 					    0, reopen_dpys);
@@ -1864,7 +2011,7 @@ if (db > 1) fprintf(stderr, "=-=   reg-scroll 0x%lx 0x%lx\n", rc_scroll, rcs_scr
 
 if (db) fprintf(stderr, "rc_scroll: 0x%lx\n", rc_scroll);
 	if (trapped_record_xerror) {
-		RECORD_ERROR_MSG;
+		RECORD_ERROR_MSG("register");
 	}
 
 	if (! rc_scroll) {
@@ -1913,7 +2060,7 @@ if (db) fprintf(stderr, "rc_scroll: 0x%lx\n", rc_scroll);
 			rfbLog("failed to enable RECORD context "
 			    "rc_scroll: 0x%lx rc: %d\n", rc_scroll, rc);
 			if (trapped_record_xerror) {
-				RECORD_ERROR_MSG;
+				RECORD_ERROR_MSG("enable-failed");
 			}
 		}
 		shutdown_record_context(rc_scroll, 0, reopen_dpys);
